﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProtoBuf;
using Protocols;
using System.IO;

namespace GameLib
{
    //完整协议体
    public class ProtoMsg
    {
        public byte[] buffer;
        public MSGID MsgId
        {
            get
            {
                return (MSGID)CmdId;
            }
        }

        public UInt16 CmdId { get; set; }

        public UInt32 BodyLen { get; set; }

        public ProtoBody protoBody;

        /// <summary>
        /// from ProtoBody to Protocol
        /// </summary>
        /// <param name="addr"></param>
        /// <returns></returns>
        public static implicit operator ProtoMsg(ProtoBody bodyIn)
        {
            return new ProtoMsg(bodyIn);
        }

        public Type BodyType
        {
            get
            {
                return ProtoID.GetType(CmdId);
            }
        }

        /// <summary>
        /// 仅生成ProtoHead
        /// </summary>
        public ProtoMsg()
        {
            protoBody = null;
        }

        /// <summary>
        /// 生成ProtoHead,ServerHead,ProtoID,并使用传入的ProtoBody赋值
        /// </summary>
        /// <param name="bodyIn"></param>
        public ProtoMsg(ProtoBody bodyIn)
        {
            protoBody = bodyIn;

            CmdId = ProtoID.GetId(bodyIn.GetType());
        }

        public byte[] ToBuffer()
        {
            using (MemoryStream stream = new MemoryStream())
            {
                ProtoBuf.Serializer.NonGeneric.Serialize(stream, protoBody);
                BodyLen = (UInt32)stream.Length;
                int bodyLen = (int)BodyLen;
                byte[] buffer = new byte[bodyLen + 6];
                stream.Position = 0;
                stream.Read(buffer, 6, bodyLen);
                stream.Position = 0;

                using (MemoryStream stream2 = new MemoryStream())
                {
                    stream2.Write(BitConverter.GetBytes(CmdId), 0, sizeof(UInt16));
                    stream2.Write(BitConverter.GetBytes(bodyLen), 0, sizeof(UInt32));
                    stream2.Position = 0;

                    stream2.Read(buffer, 0, (int)stream2.Length);
                }

                return buffer;
            }
        }

        public static ProtoMsg ParseFromBuffer(byte[] buffer)
        {
            var msgId = BitConverter.ToUInt16(buffer, 0);
            var bodyLen = BitConverter.ToUInt32(buffer, 2);

            ProtoMsg proto = new ProtoMsg();
            proto.CmdId = msgId;
            proto.BodyLen = bodyLen;

            using (var protoBodyStream = new MemoryStream())
            {
                protoBodyStream.Write(buffer, 6, buffer.Length - 6);

                protoBodyStream.Position = 0;

                proto.protoBody = Serializer.NonGeneric.Deserialize(proto.BodyType, protoBodyStream) as ProtoBody;
            }

            return proto;
        }
    }
}
